@@ -1,4 +1,4 @@
-/*	$Id: ort.h,v 1.96 2021/06/09 09:17:33 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2017--2020 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -170,11 +170,11 @@ struct	field {
 	struct strct	  *parent;
 	struct fvalidq	   fvq;
 	unsigned int	   flags;
-#define	FIELD_ROWID	   0x01
-#define	FIELD_UNIQUE	   0x02
-#define FIELD_NULL	   0x04
-#define	FIELD_NOEXPORT	   0x08
-#define FIELD_HASDEF	   0x10
+#define	FIELD_ROWID	   0x01u
+#define	FIELD_UNIQUE	   0x02u
+#define FIELD_NULL	   0x04u
+#define	FIELD_NOEXPORT	   0x08u
+#define FIELD_HASDEF	   0x10u
 	TAILQ_ENTRY(field) entries;
 };
 
@@ -357,7 +357,7 @@ struct	search {
 	int64_t		    offset;
 	struct rolemap	   *rolemap;
 	unsigned int	    flags; 
-#define	SEARCH_IS_UNIQUE    0x01
+#define	SEARCH_IS_UNIQUE    0x01u
 	TAILQ_ENTRY(search) entries;
 };
 
@@ -400,7 +400,7 @@ struct	update {
 	struct strct	   *parent;
 	struct rolemap	   *rolemap;
 	unsigned int	    flags;
-#define	UPDATE_ALL	    0x01
+#define	UPDATE_ALL	    0x01u
 	TAILQ_ENTRY(update) entries;
 };
 
@@ -427,10 +427,10 @@ struct	strct {
 	struct insert	  *ins;
 	struct rolemap	  *arolemap; /* during linkage (XXX: remove) */
 	unsigned int	   flags;
-#define	STRCT_HAS_QUEUE	   0x01
-#define	STRCT_HAS_ITERATOR 0x02
-#define	STRCT_HAS_BLOB	   0x04
-#define STRCT_HAS_NULLREFS 0x10
+#define	STRCT_HAS_QUEUE	   0x01u
+#define	STRCT_HAS_ITERATOR 0x02u
+#define	STRCT_HAS_BLOB	   0x04u
+#define STRCT_HAS_NULLREFS 0x10u
 	struct config	  *cfg;
 	TAILQ_ENTRY(strct) entries;
 };
@@ -473,6 +473,8 @@ struct	config {
 	size_t		  fnamesz;
 	struct msgq	  mq;
 	struct config_private *priv;
+	unsigned int	  flags;
+#define	CONFIG_HAS_PASS	  0x01u
 };
 
 enum	difftype {
@@ -684,7 +686,7 @@ int		 ort_write_msg_file(FILE *f, const struct msgq *);
 struct diffq	*ort_diff(const struct config *, const struct config *);
 void		 ort_diffq_free(struct diffq *);
 int		 ort_write_diff_file(FILE *, const struct diffq *,
-			const char **, size_t, const char **, size_t);
+			const char *const *, size_t, const char *const *, size_t);
 void	 	 ort_msgv(struct msgq *, enum msgtype, int, 
 			const struct pos *, const char *, va_list);
 void	 	 ort_msg(struct msgq *, enum msgtype, int, 
