@@ -61,6 +61,9 @@ namespace ortJson {
 
 	/**
 	 * Same as "struct bitidx" in ort(3).
+	 * The "mask" extension is the 64-bit number produced by the
+	 * value.
+	 * It is informational.
 	 */
 	export interface bitIndexObj {
 		parent: string;
@@ -68,6 +71,7 @@ namespace ortJson {
 		pos: posObj;
 		doc: string|null;
 		value: string|number;
+		mask: string|number;
 		labels: labelSet;
 	}
 
@@ -191,6 +195,9 @@ namespace ortJson {
 	export interface sentObj {
 		pos: posObj;
 		fname: string;
+		uname: string;
+		field: fieldPtrObj;
+		chain: string[];
 		op: sentObjOp;
 	}
 
@@ -202,6 +209,7 @@ namespace ortJson {
 	export interface orderObj {
 		pos: posObj;
 		fname: string;
+		chain: string[];
 		op: orderObjOp;
 	}
 
@@ -211,6 +219,7 @@ namespace ortJson {
 	export interface aggrObj {
 		pos: posObj;
 		fname: string;
+		chain: string[];
 		op: 'minrow'|'maxrow';
 	}
 
@@ -219,6 +228,7 @@ namespace ortJson {
 	 */
 	export interface groupObj {
 		pos: posObj;
+		chain: string[];
 		fname: string;
 	}
 
@@ -227,7 +237,9 @@ namespace ortJson {
 	 */
 	export interface dstnctObj {
 		pos: posObj;
+		chain: string[];
 		fname: string;
+		strct: string;
 	}
 
 	/**
@@ -411,6 +423,32 @@ namespace ortJson {
 		bitfs: string;
 	}
 
+	/**
+	 * These callbacks are adding custom manipulation of the DOM for
+	 * specific types of objects.  These callbacks are invoked after all
+	 * other processing for the object (e.g., the field) has run.
+	 */
+	export interface ortJsonConfigCallbacks {
+		strct?: (e: HTMLElement, strct: ortJson.strctObj,
+			arg?: any) => void;
+		query?: (e: HTMLElement, strct: ortJson.searchObj,
+			arg?: any) => void;
+		update?: (e: HTMLElement, strct: ortJson.updateObj,
+			arg?: any) => void;
+		field?: (e: HTMLElement, strct: ortJson.fieldObj,
+			arg?: any) => void;
+		enm?: (e: HTMLElement, strct: ortJson.enumObj,
+			arg?: any) => void;
+		eitem?: (e: HTMLElement, strct: ortJson.enumItemObj,
+			arg?: any) => void;
+		bitf?: (e: HTMLElement, strct: ortJson.bitfObj,
+			arg?: any) => void;
+		bitindex?: (e: HTMLElement, strct: ortJson.bitIndexObj,
+			arg?: any) => void;
+		role?: (e: HTMLElement, strct: ortJson.roleObj,
+			arg?: any) => void;
+	}
+
 	/**
 	 * Manage serialising an ortJsonConfig object into an HTML DOM
 	 * tree.
@@ -881,51 +919,52 @@ namespace ortJson {
 
 		/**
 		 * Invokes fill() for each element with the given class
-		 * under the root.
+		 * (inclusive) under the root.
 		 */
-		fillByClass(root: HTMLElement|string, name: string): void
+		fillByClass(root: HTMLElement|string, name: string,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			const e: HTMLElement|null = this.find(root);
 			if (e === null)
 				return;
 			const list: HTMLElement[] = this.list(e, name);
 			for (let i: number = 0; i < list.length; i++)
-				this.fill(list[i]);
+				this.fill(list[i], cb, arg);
 		}
 
 		/**
-		 * Fills the configuration **including** the given element,
-		 * starting with structures, then drilling down.
-		 * Elements are manipulated by whether the contain the
-		 * following classes.  Note that to "hide" an element
-		 * means to append the *hide* class to its class list,
-		 * while to "show" an element is to remove this.  The
-		 * caller will need to make sure that these classes
-		 * contain the appropriate `display: none` or similar
-		 * style definitions.
+		 * Fills the configuration the given element (inclusive),
+		 * starting with structures, then drilling down.  Elements are
+		 * manipulated by whether the contain the following classes.  To
+		 * "hide" an element means to append the *hide* class to its
+		 * class list, while to "show" an element is to remove this.
+		 * The caller will need to make sure that these classes contain
+		 * the appropriate `display: none` or similar style definitions.
 		 *
-		 * Per configuration:
+		 * - fillStrctSet() (for structs)
+		 * - fillRoleSet() (for roles)
+		 * - fillEnumSet() (for enums)
+		 * - fillBitfSet() (for bitfields)
 		 *
-		 * - fillStrctSet()
-		 * - fillRoleSet()
 		 * - *config-strcts-prefix-link*: sets 'href' to the strcts
-		 *   prefix fragment (even if empty)
+		 *   prefix fragment, even if empty
 		 * - *config-roles-prefix-link*: sets 'href' to the roles
-		 *   prefix fragment (even if empty)
+		 *   prefix fragment, even if empty
 		 * - *config-enums-prefix-link*: sets 'href' to the enums
-		 *   prefix fragment (even if empty)
+		 *   prefix fragment, even if empty
 		 * - *config-bitfs-prefix-link*: sets 'href' to the bitfs
-		 *   prefix fragment (even if empty)
-		 * - *config-strcts-prefix-id*: sets 'id' to the strcts prefix
-		 *   (even if empty)
-		 * - *config-roles-prefix-id*: sets 'id' to the roles prefix
-		 *   (even if empty)
-		 * - *config-enums-prefix-id*: sets 'id' to the enums prefix
-		 *   (even if empty)
-		 * - *config-bitfs-prefix-id*: sets 'id' to the bitfs prefix
-		 *   (even if empty)
+		 *   prefix fragment, even if empty
+		 * - *config-strcts-prefix-id*: sets 'id' to the strcts prefix,
+		 *   even if empty
+		 * - *config-roles-prefix-id*: sets 'id' to the roles prefix,
+		 *   even if empty
+		 * - *config-enums-prefix-id*: sets 'id' to the enums prefix,
+		 *   even if empty
+		 * - *config-bitfs-prefix-id*: sets 'id' to the bitfs prefix,
+		 *   even if empty
 		 */
-		fill(e: string|HTMLElement|null): void
+		fill(e: string|HTMLElement|null,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			if (e === null)
 				return;
@@ -948,13 +987,23 @@ namespace ortJson {
 				'href', '#' + this.prefixes.enums);
 			this.attrcl(pn, 'config-bitfs-prefix-link',
 				'href', '#' + this.prefixes.bitfs);
-			this.fillRoleSet(pn);
-			this.fillStrctSet(pn);
-			this.fillEnumSet(pn);
-			this.fillBitfSet(pn);
+			this.fillRoleSet(pn, undefined, cb, arg);
+			this.fillStrctSet(pn, cb, arg);
+			this.fillEnumSet(pn, cb, arg);
+			this.fillBitfSet(pn, cb, arg);
 			this.show(pn);
 		}
 
+		/**
+		 * For a possibly-null comment string "doc" of the given name
+		 * "type" (e.g., "strct" or "field"):
+		 *
+		 * - *config-type-doc-{none,has}*: shown or hidden if "doc" is
+		 *   non-null
+		 * - *config-type-doc*: filled with "doc" or emptied if null
+		 * - *config-type-doc-value*: set *value* to "doc" or empty if
+		 *   null
+		 */
 		private fillComment(e: HTMLElement, 
 			type: string, doc: string|null): void
 		{
@@ -985,7 +1034,8 @@ namespace ortJson {
 		 * - *config-bitfs-name-id*: set *id* to 'bitfs' within the
 		 *   *config-bitfs-prefix-id* prefix
 		 */
-		fillBitfSet(e: HTMLElement): void
+		fillBitfSet(e: HTMLElement,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-bitfs-name-link', 
 				'href', '#' + this.prefixes.bitfs + 'bitfs');
@@ -1013,8 +1063,8 @@ namespace ortJson {
 					const cln: HTMLElement = 
 						<HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillBitfObj
-						(cln, this.obj.bq[name]);
+					this.fillBitfObj(cln, 
+						this.obj.bq[name], cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -1035,7 +1085,8 @@ namespace ortJson {
 		 * - *config-enums-name-id*: set *id* to 'enums' within the
 		 *   *config-enums-prefix-id* prefix
 		 */
-		fillEnumSet(e: HTMLElement): void
+		fillEnumSet(e: HTMLElement,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-enums-name-link', 
 				'href', '#' + this.prefixes.enums + 'enums');
@@ -1065,7 +1116,7 @@ namespace ortJson {
 						<HTMLElement>
 						tmpl.cloneNode(true);
 					this.fillEnumObj(cln, 
-						this.obj.eq[keys[j]]);
+						this.obj.eq[keys[j]], cb, arg);
 					list[i].appendChild(cln);
 				}
 			}
@@ -1074,7 +1125,6 @@ namespace ortJson {
 		/**
 		 * For a bitfield:
 		 *
-		 * - *config-bitf-doc*: filled in with bitfield documentation
 		 * - *config-bitf-name*: filled in with the bitfield name
 		 * - *config-bitf-name-data*: *data-name* set to the bitfield
 		 *   name
@@ -1084,13 +1134,17 @@ namespace ortJson {
 		 * - *config-bitf-name-link*: *href* set as the name within the
 		 *   *config-bitfs-name-link* prefix
 		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * For bitfield items:
 		 * - see fillBitIndexSet()
 		 *
 		 * For labels:
 		 * - see fillLabelSet() ("bitf-null" and "bitf-unset")
 		 */
-		fillBitfObj(e: HTMLElement, bitf: ortJson.bitfObj): void
+		fillBitfObj(e: HTMLElement, bitf: ortJson.bitfObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.fillComment(e, 'bitf', bitf.doc);
 			this.replcl(e, 'config-bitf-name', bitf.name);
@@ -1103,15 +1157,17 @@ namespace ortJson {
 			this.attrcl(e, 'config-bitf-name-link', 'href', 
 				'#' + this.prefixes.bitfs + 'bitfs/' + 
 				bitf.name);
-			this.fillBitIndexSet(e, bitf);
+			this.fillBitIndexSet(e, bitf, cb, arg);
 			this.fillLabelSet(e, 'bitf-null', bitf.labelsNull);
 			this.fillLabelSet(e, 'bitf-unset', bitf.labelsUnset);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.bitf !== 'undefined')
+				cb.bitf(e, bitf, arg);
 		}
 
 		/**
 		 * For an enumeration:
 		 *
-		 * - *config-enum-doc*: filled in with enum documentation
 		 * - *config-enum-name*: filled in with the enum name
 		 * - *config-enum-name-id*: *id* set as the name within the
 		 *   *config-enums-name-id* prefix
@@ -1121,13 +1177,17 @@ namespace ortJson {
 		 * - *config-enum-name-data*: *data-name* attribute set to the
 		 *   enum name
 		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * For enumeration items:
 		 * - see fillEnumItemSet() 
 		  
 		 * For labels:
 		 * - see fillLabelSet() (name "enum-null")
 		 */
-		fillEnumObj(e: HTMLElement, enm: ortJson.enumObj): void
+		fillEnumObj(e: HTMLElement, enm: ortJson.enumObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.fillComment(e, 'enum', enm.doc);
 			this.replcl(e, 'config-enum-name', enm.name);
@@ -1140,8 +1200,11 @@ namespace ortJson {
 			this.attrcl(e, 'config-enum-name-link', 'href', 
 				'#' + this.prefixes.enums + 'enums/' + 
 				enm.name);
-			this.fillEnumItemSet(e, enm);
+			this.fillEnumItemSet(e, enm, cb, arg);
 			this.fillLabelSet(e, 'enum-null', enm.labelsNull);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.enm !== 'undefined')
+				cb.enm(e, enm, arg);
 		}
 
 		/**
@@ -1157,8 +1220,8 @@ namespace ortJson {
 		 *   filled in with fillEnumItemObj() for each item unless there
 		 *   are no items, in which case the element is hidden
 		 */
-		private fillEnumItemSet(e: HTMLElement, 
-			enm: ortJson.enumObj): void
+		private fillEnumItemSet(e: HTMLElement, enm: ortJson.enumObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-eitems-name-link', 'href', 
 				'#' + this.prefixes.enums + 'enums/' + 
@@ -1185,8 +1248,8 @@ namespace ortJson {
 				for (let j = 0; j < keys.length; j++) {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillEnumItemObj
-						(cln, enm.eq[keys[j]]);
+					this.fillEnumItemObj(cln, 
+						enm.eq[keys[j]], cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -1196,7 +1259,6 @@ namespace ortJson {
 		/**
 		 * For an enumeration item:
 		 *
-		 * - *config-enumitem-doc*: set text to item docs
 		 * - *config-enumitem-name*: set text to item name
 		 * - *config-enumitem-name-value*: set *value* to item name
 		 * - *config-enumitem-name-id*: *id* set as the name within the
@@ -1209,11 +1271,15 @@ namespace ortJson {
 		 *   name, if applicable, or an empty string of unset
 		 * - *config-enumitem-value-value*: *value* set to item value,
 		 *   if applicable, or an empty string if unset
-		  
+		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * For labels:
 		 * - see fillLabelSet() (name "enumitem")
 		 */
-		fillEnumItemObj(e: HTMLElement, eitem: ortJson.enumItemObj): void
+		fillEnumItemObj(e: HTMLElement, eitem: ortJson.enumItemObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.fillComment(e, 'enumitem', eitem.doc);
 			this.attrcl(e, 'config-enumitem-name-id', 'id', 
@@ -1240,6 +1306,9 @@ namespace ortJson {
 					'value', '');
 			}
 			this.fillLabelSet(e, 'enumitem', eitem.labels);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.eitem !== 'undefined')
+				cb.eitem(e, eitem, arg);
 		}
 
 		/**
@@ -1304,8 +1373,8 @@ namespace ortJson {
 		 *   filled in with fillBitIndexObj() for each item unless there
 		 *   are no items, in which case the element is hidden
 		 */
-		private fillBitIndexSet(e: HTMLElement, 
-			bitf: ortJson.bitfObj): void
+		private fillBitIndexSet(e: HTMLElement, bitf: ortJson.bitfObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-bitems-name-link', 'href', 
 				'#' + this.prefixes.bitfs + 'bitfs/' + 
@@ -1331,8 +1400,8 @@ namespace ortJson {
 				for (let j = 0; j < keys.length; j++) {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillBitIndexObj
-						(cln, bitf.bq[keys[j]]);
+					this.fillBitIndexObj(cln,
+						bitf.bq[keys[j]], cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -1346,16 +1415,20 @@ namespace ortJson {
 		 *   *config-bitfs-name-id* prefix
 		 * - *config-bitindex-name-link*: *href* set as the name within the
 		 *   *config-bitfs-name-link* prefix
-		 * - *config-bitindex-doc*: set text to item documentation
 		 * - *config-bitindex-name*: set text to item name
 		 * - *config-bitindex-name-value*: *value* set to name
 		 * - *config-bitindex-value*: set text to item value
 		 * - *config-bitindex-value-value*: *value* set to item value
-		  
+		 * - *config-bitindex-mask*: set text to item mask
+		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * For labels:
 		 * - see fillLabelSet() (name "bitindex")
 		 */
-		fillBitIndexObj(e: HTMLElement, biti: ortJson.bitIndexObj): void
+		fillBitIndexObj(e: HTMLElement, biti: ortJson.bitIndexObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.fillComment(e, 'bitindex', biti.doc);
 			this.attrcl(e, 'config-bitindex-name-id', 'id', 
@@ -1367,11 +1440,16 @@ namespace ortJson {
 			this.replcl(e, 'config-bitindex-name', biti.name);
 			this.replcl(e, 'config-bitindex-value', 
 				biti.value.toString());
+			this.replcl(e, 'config-bitindex-mask', 
+				biti.mask.toString());
 			this.attrcl(e, 'config-bitindex-name-value', 
 				'value', biti.name);
 			this.attrcl(e, 'config-bitindex-value-value', 
 				'value', biti.value.toString());
 			this.fillLabelSet(e, 'bitindex', biti.labels);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.bitindex !== 'undefined')
+				cb.bitindex(e, biti, arg);
 		}
 
 		/**
@@ -1390,7 +1468,8 @@ namespace ortJson {
 		 *
 		 * Accepts an optional array of role names not to show.
 		 */
-		fillRoleSet(e: HTMLElement, exclude?: string[]): void
+		fillRoleSet(e: HTMLElement, exclude?: string[],
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-roles-name-link', 
 				'href', '#' + this.prefixes.roles + 'roles');
@@ -1426,7 +1505,7 @@ namespace ortJson {
 					const cln: HTMLElement = 
 						<HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillRoleObj(cln, newrq[j]);
+					this.fillRoleObj(cln, newrq[j], cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -1436,7 +1515,6 @@ namespace ortJson {
 		/**
 		 * For each role:
 		 *
-		 * - *config-role-doc*: filled in with role documentation
 		 * - *config-role-name*: filled in with the role name
 		 * - *config-role-name-value*: value set to the role name
 		 * - *config-role-name-data*: 'data-name' attribute set with
@@ -1456,8 +1534,12 @@ namespace ortJson {
 		 * - *config-role-subrq*: clone each element beneath for each
 		 *   subrole and fill in the *config-role-subrq-role-name* and
 		 *   *config-role-subrq-role-link* (or hide if none)
+		 *
+		 * For documentation:
+		 * - see fillComment()
 		 */
-		fillRoleObj(e: HTMLElement, role: ortJson.roleObj): void
+		fillRoleObj(e: HTMLElement, role: ortJson.roleObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.fillComment(e, 'role', role.doc);
 			this.replcl(e, 'config-role-name', role.name);
@@ -1487,37 +1569,40 @@ namespace ortJson {
 				this.hidecl(e, 'config-role-subrq-has');
 				this.replcl(e, 'config-role-subrq-join', '');
 				this.hidecl(e, 'config-role-subrq');
-				return;
-			} 
-			this.hidecl(e, 'config-role-subrq-none');
-			this.showcl(e, 'config-role-subrq-has');
-			this.replcl(e, 'config-role-subrq-join', 
-				role.subrq.join(', '));
-			const list: HTMLElement[] = 
-				this.list(e, 'config-role-subrq');
-			for (let i: number = 0; i < list.length; i++) {
-				if (list[i].children.length === 0)
-					continue;
-				const tmpl: HTMLElement =
-					<HTMLElement>list[i].children[0];
-				this.clr(list[i]);
-				for (let j: number = 0; 
-			  	     j < role.subrq.length; j++) {
-					const cln: HTMLElement = 
-						<HTMLElement>
-						tmpl.cloneNode(true);
-					this.replcl(cln, 
-						'config-role-subrq-role-name', 
-						role.subrq[j]);
-					this.attrcl(cln, 
-						'config-role-subrq-role-link', 
-						'href', '#' + 
-						this.prefixes.roles + 
-						'roles/' + role.subrq[j]);
-					list[i].appendChild(cln);
+			} else { 
+				this.hidecl(e, 'config-role-subrq-none');
+				this.showcl(e, 'config-role-subrq-has');
+				this.replcl(e, 'config-role-subrq-join', 
+					role.subrq.join(', '));
+				const list: HTMLElement[] = 
+					this.list(e, 'config-role-subrq');
+				for (let i: number = 0; i < list.length; i++) {
+					if (list[i].children.length === 0)
+						continue;
+					const tmpl: HTMLElement =
+						<HTMLElement>list[i].children[0];
+					this.clr(list[i]);
+					for (let j: number = 0; 
+					     j < role.subrq.length; j++) {
+						const cln: HTMLElement = 
+							<HTMLElement>
+							tmpl.cloneNode(true);
+						this.replcl(cln, 
+							'config-role-subrq-role-name',
+							role.subrq[j]);
+						this.attrcl(cln, 
+							'config-role-subrq-role-link', 
+							'href', '#' + 
+							this.prefixes.roles + 
+							'roles/' + role.subrq[j]);
+						list[i].appendChild(cln);
+					}
+					this.show(list[i]);
 				}
-				this.show(list[i]);
 			}
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.role !== 'undefined')
+				cb.role(e, role, arg);
 		}
 
 		/**
@@ -1534,7 +1619,8 @@ namespace ortJson {
 		 * - *config-strcts-name-id*: set *id* to 'strcts' within the
 		 *   *config-strcts-prefix-id* prefix
 		 */
-		fillStrctSet(e: HTMLElement)
+		fillStrctSet(e: HTMLElement,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-strcts-name-link', 
 				'href', '#' + this.prefixes.strcts + 'strcts');
@@ -1559,10 +1645,12 @@ namespace ortJson {
 					<HTMLElement>list[i].children[0];
 				this.clr(list[i]);
 				for (let j: number = 0; j < keys.length; j++) {
+					const strct: ortJson.strctObj =
+						this.obj.sq[keys[j]];
 					const cln: HTMLElement = 
 						<HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillStrctObj(cln, this.obj.sq[keys[j]]);
+					this.fillStrctObj(cln, strct, cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -1580,12 +1668,6 @@ namespace ortJson {
 		 * - *config-strct-name-value*: value set with name
 		 * - *config-strct-name-data*: *data-name* attribute set with
 		 *   name
-		 * - *config-strct-doc-{none,has}*: shown or hidden depending on
-		 *   whether there's a non-empty documentation field
-		 * - *config-strct-doc*: filled in with non-empty documentation,
-		 *   if found
-		 * - *config-strct-doc-value*: value set with non-empty
-		 *   documentation, if found
 		 * - *config-fields*: the first child of this is cloned and
 		 *   filled in with data for each field (see "Per field")
 		 * - *config-insert-{has,none}*: shown or hidden depending on
@@ -1598,6 +1680,9 @@ namespace ortJson {
 		 *   unique") unless there are no tuples, in which case the
 		 *   element is hidden
 		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * For any insert:
 		 * - see fillRolemap()
 		 *
@@ -1613,9 +1698,10 @@ namespace ortJson {
 		 * For queries:
 		 * - see fillSearchClassObj() 
 		 */
-		fillStrctObj(e: HTMLElement, strct: ortJson.strctObj): void
+		fillStrctObj(e: HTMLElement, strct: ortJson.strctObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
-			this.fillFieldSet(e, strct);
+			this.fillFieldSet(e, strct, undefined, cb, arg);
 
 			if (strct.nq.length) {
 				this.showcl(e, 'config-uniques-has');
@@ -1629,9 +1715,9 @@ namespace ortJson {
 				this.showcl(e, 'config-uniques-none');
 			}
 
-			this.fillUpdateClassObj(e, 'deletes', strct, strct.dq);
-			this.fillUpdateClassObj(e, 'updates', strct, strct.uq);
-			this.fillSearchClassObj(e, strct);
+			this.fillUpdateClassObj(e, 'deletes', strct, strct.dq, cb, arg);
+			this.fillUpdateClassObj(e, 'updates', strct, strct.uq, cb, arg);
+			this.fillSearchClassObj(e, strct, cb, arg);
 
 			if (strct.insert !== null) {
 				this.showcl(e, 'config-insert-has');
@@ -1655,6 +1741,9 @@ namespace ortJson {
 			this.attrcl(e, 'config-strct-name-link', 'href', 
 				'#' + this.prefixes.strcts + 'strcts/' + 
 				strct.name);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.strct !== 'undefined')
+				cb.strct(e, strct, arg);
 		}
 
 		/**
@@ -1742,7 +1831,8 @@ namespace ortJson {
 		 */
 		private fillUpdateClassObj(e: HTMLElement, 
 			type: 'updates'|'deletes', strct: ortJson.strctObj,
-			cls: ortJson.updateClassObj): void
+			cls: ortJson.updateClassObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-' + type + '-name-link', 'href', 
 				'#' + this.prefixes.strcts + 'strcts/' + 
@@ -1772,16 +1862,17 @@ namespace ortJson {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
 					this.fillUpdateObj(cln, type,
-						cls.named[keys[j]], j);
+						cls.named[keys[j]], j, cb, arg);
 					list[i].appendChild(cln);
 				}
 				for (let j = 0; j < cls.anon.length; j++) {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
 					this.fillUpdateObj(cln, type,
-						cls.anon[j], j);
+						cls.anon[j], j, cb, arg);
 					list[i].appendChild(cln);
 				}
+				this.show(list[i]);
 			}
 		}
 
@@ -1789,10 +1880,6 @@ namespace ortJson {
 		 * Per update (or delete):
 		 *
 		 * - see fillRolemap()
-		 * - *config-update-doc-{has,none}*: shown or hidden depending
-		 *   on whether there's a non-empty documentation field
-		 * - *config-update-doc*: filled in with non-empty documentation
-		 *   if doc is defined
 		 * - *config-update-name-{has,none}*: shown or hidden depending
 		 *   on whether the update is anonymous
 		 * - *config-update-name*: filled in with the update name if a
@@ -1820,6 +1907,9 @@ namespace ortJson {
 		 *   reference") unless there are no references, in which case
 		 *   the element is hidden
 		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * Per update reference:
 		 * - *config-uref-field*: the field name in the current
 		 *   structure
@@ -1832,7 +1922,8 @@ namespace ortJson {
 		 * - *config-uref-mod-value*: value set to the update modifier
 		 */
 		fillUpdateObj(e: HTMLElement, type: 'updates'|'deletes',
-			up: ortJson.updateObj, pos: number): void
+			up: ortJson.updateObj, pos: number,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.fillComment(e, 'update', up.doc);
 			if (up.name === null) {
@@ -1898,6 +1989,9 @@ namespace ortJson {
 					this.fillUrefs(list[i], up.mrq);
 			}
 			this.fillRolemap(e, 'update', up.rolemap);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.update !== 'undefined')
+				cb.update(e, up, arg);
 		}
 
 		private fillUrefs(e: HTMLElement, 
@@ -1942,7 +2036,8 @@ namespace ortJson {
 		 *   hidden
 		 */
 		private fillSearchClassObj(e: HTMLElement, 
-			strct: ortJson.strctObj): void
+			strct: ortJson.strctObj, 
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			this.attrcl(e, 'config-queries-name-link', 'href', 
 				'#' + this.prefixes.strcts + 'strcts/' + 
@@ -1972,13 +2067,15 @@ namespace ortJson {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
 					this.fillSearchObj(cln, 
-						strct.sq.named[keys[j]], j);
+						strct.sq.named[keys[j]], j, 
+						cb, arg);
 					list[i].appendChild(cln);
 				}
 				for (let j = 0; j < strct.sq.anon.length; j++) {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillSearchObj(cln, strct.sq.anon[j], j);
+					this.fillSearchObj(cln, strct.sq.anon[j],
+						j, cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -2001,11 +2098,6 @@ namespace ortJson {
 		 *   during construction (or _query + unique number)
 		 * - *config-query-name-link*: *href* set as the same fragment
 		 *   set in *config-query-name-id*
-		 * - *config-query-doc-{has,none}*: shown or hidden
-		 *   depending on whether there's a non-empty
-		 *   documentation field
-		 * - *config-query-doc*: filled in with non-empty
-		 *   documentation if doc is defined*
 		 * - *config-query-type*: filled in with the query type
 		 * - *config-query-sntq-{has,none}*: whether there's a
 		 *   list of search columns
@@ -2043,6 +2135,9 @@ namespace ortJson {
 		 * - *config-query-limit-value*: value filled in with
 		 *   the limit
 		 *
+		 * For documentation:
+		 * - see fillComment()
+		 *
 		 * Per query search field:
 		 * - *config-sent-fname*: filled in with the fname
 		 * - *config-sent-fname-value*: value filled in with the
@@ -2060,7 +2155,8 @@ namespace ortJson {
 		 *   operation
 		 */
 		fillSearchObj(e: HTMLElement, query: ortJson.searchObj,
-			pos: number): void
+			pos: number, cb?: ortJson.ortJsonConfigCallbacks,
+			arg?: any): void
 		{
 			this.fillComment(e, 'query', query.doc);
 			if (query.name === null) {
@@ -2169,6 +2265,9 @@ namespace ortJson {
 				this.showcl(e, 'config-query-aggr-none');
 			}
 			this.fillRolemap(e, 'query', query.rolemap);
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.query !== 'undefined')
+				cb.query(e, query, arg);
 		}
 
 		private fillOrds(e: HTMLElement, 
@@ -2231,7 +2330,8 @@ namespace ortJson {
 		 *   are no fields, in which case the elements are hidden
 		 */
 		fillFieldSet(e: HTMLElement, strct: ortJson.strctObj,
-			exclude?: string[]): void
+			exclude?: string[], cb?: ortJson.ortJsonConfigCallbacks,
+			arg?: any): void
 		{
 			this.attrcl(e, 'config-fields-name-link', 'href', 
 				'#' + this.prefixes.strcts + 'strcts/' + 
@@ -2264,8 +2364,8 @@ namespace ortJson {
 				for (let j: number = 0; j < keys.length; j++) {
 					const cln: HTMLElement = <HTMLElement>
 						tmpl.cloneNode(true);
-					this.fillFieldObj
-						(cln, strct.fq[keys[j]]);
+					this.fillFieldObj(cln, 
+						strct.fq[keys[j]], cb, arg);
 					list[i].appendChild(cln);
 				}
 				this.show(list[i]);
@@ -2296,10 +2396,6 @@ namespace ortJson {
 		 *   *config-fields-name-link* prefix
 		 * - *config-field-fullname-data*: 'data-fullname' attribute set
 		 *   as "parent.name"
-		 * - *config-field-doc-{none,has}*: shown or hidden depending on
-		 *   whether there's a non-empty documentation field
-		 * - *config-field-doc*: filled in with non-empty documentation
-		 *   if doc is defined
 		 * - *config-field-type-TYPE*: shown or hidden depending upon
 		 *   the field type
 		 * - *config-field-type*: filled in with the type name
@@ -2359,8 +2455,12 @@ namespace ortJson {
 		 *   on whether the field has flags
 		 * - *config-field-flags-join*: filled in with comma-separated
 		 *   flags or empty string if having none
+		 *
+		 * For documentation:
+		 * - see fillComment()
 		 */
-		fillFieldObj(e: HTMLElement, field: ortJson.fieldObj): void
+		fillFieldObj(e: HTMLElement, field: ortJson.fieldObj,
+			cb?: ortJson.ortJsonConfigCallbacks, arg?: any): void
 		{
 			const flags: fieldObjFlags[] = [
 				'rowid', 'null', 'unique', 'noexport'
@@ -2531,6 +2631,10 @@ namespace ortJson {
 				}
 				this.replcl(e, 'config-field-limit', lim);
 			}
+
+			if (typeof cb !== 'undefined' &&
+			    typeof cb.field !== 'undefined')
+				cb.field(e, field, arg);
 		}
 	}
 }
